# Warframe One-Click Scripts

This repo provides Windows batch scripts for easy backup and deployment of your Warframe server and MongoDB database.  
All required dependencies (MongoDB, Node.js, Git, etc.) are automatically downloaded and managed in the `external` directory.

---

## OneClickDeploy.cmd

Download, update, and launch the Warframe server, MongoDB, and IRC server.

### Usage

```bat
OneClickDeploy.cmd [options] [Positional Arguments]
```

#### Options

- `-externalDir [path]`  
  Set the external tools directory (default: `<script directory>\external`)

- `-serverDir [path]`  
  Set the server code directory (default: `<script directory>\SpaceNinjaServer`)

- `-serverRepo [url]`  
  Set the server repository URL (default: `https://openwf.io/SpaceNinjaServer.git`)

- `-databasePath [path]`  
  Set the MongoDB data directory (default: `externalDir\mongodb-data`)

- `-ignorePull`  
  Ignore pulling updates for the server repository

- `-irc-options [args]`  
  Options to pass to the IRC server executable

- `-db-options [args]`  
  Options to pass to the MongoDB server executable

- `-server-options [args]`  
  Options to pass to the main server

#### Positional Arguments

- `database` — Launch the database server
- `irc` — Launch the IRC server
- `server` — Launch the main server

If no positional arguments are provided, **all servers will be launched**.

#### Example

Launch everything (default):

```bat
OneClickDeploy.cmd
```

Launch only the main server:

```bat
OneClickDeploy.cmd server
```

Specify a custom server directory and MongoDB port:

```bat
OneClickDeploy.cmd -serverDir="D:\MyServer" -db-options="--port 30002"
```

---

## OneClickBackup.cmd

Backup and optionally restore your MongoDB database with a single command.

### Usage

```bat
OneClickBackup.cmd [options] [backup-archive(optional)]
```

#### Options

- `-externalDir [path]`  
  Set the external tools directory (default: `<script directory>\external`)

- `-backupDir [path]`  
  Set the backup directory (default: `<script directory>\backup`)

- `-mongodbUri [uri]`  
  Set the MongoDB URI (default: `mongodb://127.0.0.1:27017/openWF`)

#### Positional Argument

- `[backup-archive]`  
  (Optional) Path to a backup archive file. If provided and exists, the script will restore the database from this archive after backup.

#### Example

```bat
OneClickBackup.cmd -backupDir="D:\backups" -mongodbUri="mongodb://localhost:27017/openWF"
```

To restore from a backup archive:

```bat
OneClickBackup.cmd D:\backups\2024-06-09_12-00-00.gz
```

---

## Notes

- The scripts will automatically download and set up required tools if missing.
- All logs and backups are stored in the specified directories.
- First run may take several minutes to download dependencies.
- You can customize all paths and options via command-line arguments.

---

## Troubleshooting

- If you encounter errors, check the `.backup_log.txt` in your backup directory for details.
- Make sure you have internet access for the initial setup.
